/* NAME:
 *	s2av - enargv a string
 *
 * SYNOPSIS:
 *	char **s2av(char *s, const char *tok, int *acp)
 *
 * DESCRIPTION:
 *	
 *	
 * AUTHOR:
 *	Simon J. Gerraty <sjg@quick.com.au>
 */
/*
 *      @(#)Copyright (c) 1994, Simon J. Gerraty.
 *      
 *      This is free software.  It comes with NO WARRANTY.
 *      Permission to use, modify and distribute this source code 
 *      is granted subject to the following conditions.
 *      1/ that the above copyright notice and this notice 
 *      are preserved in all copies and that due credit be given 
 *      to the author.  
 *      2/ that any changes to this code are clearly commented 
 *      as such so that the author does not get blamed for bugs 
 *      other than his own.
 *      
 *      Please send copies of changes and bug-fixes to:
 *      sjg@quick.com.au
 */

#ifndef lint
static char RCSid[] = "$Id: s2av.c,v 1.7 1998/05/20 09:54:47 sjg Exp $";
#endif
#include <sys/cdefs.h>
#include <stdio.h>
#ifdef __STDC__
#include <stdlib.h>
#endif
#include <sys/param.h>
#include <sys/types.h>
#include <string.h>

#if defined(HAVE_DMALLOC_H) && defined(USE_DMALLOC)
# include "dmalloc.h"
#else
# ifdef HAVE_MALLOC_H
#   include "malloc.h"
# endif
#endif

#ifndef S2AV_MIN
# define S2AV_MIN 10
#endif
#ifndef S2AV_NEXT
# define S2AV_NEXT 10
#endif

char **
s2avb(s, tok, base, acp)
     char *s;
     const char *tok;
     int base;
     int *acp;
{
	char **av;
	int i = 0, n = S2AV_MIN;

	if (acp)
		*acp = 0;
	if (s == 0 || *s == 0) {
		return 0;
	}

	av = (char **) malloc(S2AV_MIN * sizeof(char *));

	if (av) {
		do {
			av[i++] = s;
		} while (i <= base && i < n);
		if (strtok(s, tok)) {
			while (av[i++] = strtok(0, tok)) {
				if (i == n) {
					n += S2AV_NEXT;
					av = (char **) realloc(av, n * sizeof(char *));
				}
			}
		} else 
			av[i] = 0;
		if (acp)
			*acp = i - 1;
	}
	return av;
}


char **
s2av(s, tok, acp)
     char *s;
     const char *tok;
     int *acp;
{
	return s2avb(s, tok, 0, acp);
}

#ifdef MAIN

int
main(argc, argv)
	int argc;
	char **argv;
{
	char *toks = ":,";
	char **av;
	char *cp;
	int ac;
	int i;

	if (argc > 2)
		toks = argv[2];
	cp = (argc > 1) ? strdup(argv[1]) : strdup("one,two,three");
	av = s2avb(cp, toks, argc - 3, &ac);
	for (i = 0; i < ac; i++)
		printf("av[%d]='%s'\n", i, av[i]);
	free(av);
	free(cp);
	exit(0);
}
#endif
	
