/*
 * NAME:
 *	utcsecscmp - compare X509 date with time
 *
 * SYNOPSIS:
 *	int utcSecsCmp(ASN1_UTCTIME *"utm", time_t "secs", time_t "adjsecs")
 *
 * DESCRIPTION:
 *	This is a variation on the X509_cmp_current_time() in SSLeay.
 *	The difference is that we can compare against an arbitrary
 *	time which is handy for warning if a cert will expire within
 *	say 30 days.
 *	
 * AUTHOR:
 *	Simon J. Gerraty <sjg@quick.com.au>
 */
/*
 *      @(#)Copyright (c) 1996 Simon J. Gerraty.
 *      
 *      This is free software.  It comes with NO WARRANTY.
 *      Permission to use, modify and distribute this source code 
 *      is granted subject to the following conditions.
 *      1/ that the above copyright notice and this notice 
 *      are preserved in all copies and that due credit be given 
 *      to the author.  
 *      2/ that any changes to this code are clearly commented 
 *      as such so that the author does not get blamed for bugs 
 *      other than his own.
 *      
 *      Please send copies of changes and bug-fixes to:
 *      sjg@quick.com.au
 */
#ifndef lint
static char RCSid[] = "$Id: utcsecscmp.c,v 1.1 1998/02/27 06:19:49 sjg Exp $";
#endif

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <string.h>

#include "asn1.h"

 
int
utcSecsCmp(utm, secs, adjsecs)
	ASN1_UTCTIME *utm;
	time_t secs, adjsecs;
{
	ASN1_UTCTIME x;
	time_t offset = 0;
	char b1[24], b2[24], *s, *p;
	int i, j;
	
	if ((i = utm->length) < 11 || i > 17)
		return 0;
	
	if (!secs)
		time(&secs);
	if (adjsecs)
		secs += adjsecs;

	/*
	 * now we may also need to adjust for a
	 * timezone offset in utm...
	 */
	s = (char *)utm->data;
	p = (char *)memcpy(b1, s, 10);
	p += 10;
	s += 10;
	
	if (*s == 'Z' || *s == '+' || *s == '-') {
		*p++ = '\0';
		*p++ = '\0';
	} else {
		*p++ = *s++;
		*p++ = *s++;
	}
	*p++ = 'Z';
	*p++ = '\0';
	if (*s != 'Z') {
		if (*s != '+' && *s != '-')
			return 0;
		offset = ((s[1] - '0') * 10 + (s[2] - '0')) * 60;
		if (*s == '-')
			 /* I think X509_cmp_current_time() is wrong */
			offset = -offset;
	}
	if (offset)
		secs += offset;
	
	x.type = V_ASN1_UTCTIME;
	x.length = sizeof(b2);
	x.data = (unsigned char *)b2;
	ASN1_UTCTIME_set(&x, secs);

	/*
	 * now compare them... as per X509_cmp_current_time()
	 */
	i = (b1[0] - '0') * 10 + (b1[1] - '0');
	if (i < 70)
		i += 100;
	j = (b2[0] - '0') * 10 + (b2[1] - '0');
	if (j < 70)
		j += 100;
	if (i < j)
		return (-1);
	if (i > j)
		return (1);
	i = strcmp(b1,b2);
	if (i == 0) /* wait a second then return younger :-) */
		return (-1);
	else
		return (i);
}

#ifdef MAIN
int
main(argc, argv)
	int argc;
	char *argv[];
{
	ASN1_UTCTIME x;
	char b[24];
	time_t s, a;
	int i;
	
	if (argc < 4) {
		fprintf(stderr, "usage:\n\t%s utm secs adj\n",
			argv[0]);
		exit(1);
	}
	
	x.type = V_ASN1_UTCTIME;
	x.data = (unsigned char *)b;
	strncpy(b, argv[1], sizeof(b));
	b[sizeof(b) - 1] = '\0';
	x.length = strlen(b);
	s = atol(argv[2]);
	a = atol(argv[3]);
	i = utcSecsCmp(&x, s, a);
	printf("utcSecsCmp(%s,%ld,%ld) == %d\n",
	       argv[1], s, a, i);
	exit(0);
}
#endif
